// Generated by CoffeeScript 2.2.1
(function() {
  var CRLF, Chat2, EventEmitter, eTimedOut, net;

  net = require('net');

  EventEmitter = require('events');

  CRLF = "\r\n";

  eTimedOut = function(msg) {
    var err;
    err = new Error(msg);
    err.code = 'ETIMEDOUT';
    return err;
  };

  Chat2 = (function() {
    class Chat2 {
      constructor(host, port, timeout) {
        this.host = host;
        this.port = port;
        this.timeout = timeout;
        if (this.port == null) {
          this.port = 23;
        }
        if (typeof this.host !== 'string') {
          throw new TypeError('host must be a string');
        }
        if (typeof this.port !== 'number') {
          throw new TypeError('port must be a number');
        }
        if (this.timeout != null) {
          if (!(this.timeout > 0)) {
            throw new TypeError('timeout (if defined) must be greater than zero');
          }
        }
        this.emitter = new EventEmitter().setMaxListeners(2e308);
      }

      connect() {
        return new Promise((resolve, reject) => {
          var timer;
          if (this.timeout) {
            timer = setTimeout(() => {
              var ref;
              reject(eTimedOut(`Connection timed out ${this.host}:${this.port}`));
              if ((ref = this.client) != null) {
                ref.destroy();
              }
              return delete this.client;
            }, this.timeout);
          }
          return this.client = net.createConnection({
            host: this.host,
            port: this.port
          }).setEncoding('utf-8').once('connect', () => {
            if (timer) {
              clearTimeout(timer);
            }
            return resolve(this.client.setKeepAlive(true));
          }).once('error', (err) => {
            if (timer) {
              clearTimeout(timer);
            }
            return reject(err);
          }).once('end', () => {
            return delete this.client;
          }).on('data', (data) => {
            var lines;
            data = this.pending + data;
            lines = data.split(CRLF); // lines may have a "" as the last element
            this.pending = lines.pop(); // null unless partial line
            if (lines.length) {
              this.queue = this.queue.concat(lines); // add any lines to the queue
              return this.emitter.emit('newline', lines); // and emit for the next waiting expect
            }
          });
        });
      }

      expect(pattern) {
        return new Promise((resolve, reject) => {
          var getLine, timer;
          if (this.client == null) {
            return reject(new Error('Expect error: No socket'));
          }
          if (this.timeout) {
            timer = setTimeout(() => {
              return reject(eTimedOut(`Expect timed out: ${pattern}`));
            }, this.timeout);
          }
          getLine = () => {
            var line;
            while (this.queue.length > 0) {
              line = this.queue.shift();
              if (line.match(pattern)) {
                if (timer) {
                  clearTimeout(timer);
                }
                this.emitter.removeListener('newline', getLine);
                resolve(line);
                return true; // Return true if you've resolved
              }
            }
          };
          // being careful to leave the rest of the queue
          if (!getLine()) { // Try it once and, if it fails, wait for the event
            return this.emitter.on('newline', getLine);
          }
        });
      }

      send(string) {
        return new Promise((resolve, reject) => {
          if (this.client == null) {
            return reject(new Error('Send error: No socket'));
          }
          this.client.write(string + CRLF);
          return resolve(string);
        });
      }

      exec(string, pattern) {
        if (!((string != null) && (pattern != null))) {
          return Promise.reject(new Error('string and pattern required'));
        }
        return this.send(string).then(() => {
          return this.expect(pattern);
        });
      }

      close() {
        var ref, ref1;
        if ((ref = this.client) != null) {
          ref.end();
        }
        return (ref1 = this.emitter) != null ? ref1.removeAllListeners() : void 0;
      }

    };

    Chat2.prototype.queue = [];

    Chat2.prototype.pending = '';

    return Chat2;

  }).call(this);

  // client will be deleted by the 'end' event
  module.exports = Chat2;

}).call(this);
